/////////////////////////////////////////////////////////////
// Flash Plugin and Player
// Copyright (C) 1998 Olivier Debon
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// 
///////////////////////////////////////////////////////////////
//  Author : Olivier Debon  <odebon@club-internet.fr>
//

#include <assert.h>
#include "graphic.h"
#include "text.h"

static char *rcsid = "$Id: text.cc,v 1.7 1998/11/11 14:41:10 olivier Exp $";

Text::Text(long id) : Character(id)
{
	textRecords = 0;
}

Text::~Text()
{
	TextRecord     *cur,*del;

	for(cur = textRecords; cur;)
	{
		del = cur;
		cur = cur->next;
		delete del;
	}
}

void
Text::setTextBoundary(long x1, long y1, long x2, long y2)
{
	xMin = x1;
	yMin = y1;
	xMax = x2;
	yMax = y2;
}

void
Text::setTextMatrix(Matrix m)
{
	textMatrix = m;
}

void
Text::addTextRecord(TextRecord *tr)
{
	tr->next = 0;

	if (textRecords == 0) {
		textRecords = tr;
	} else {
		TextRecord *current;

		for(current = textRecords; current->next; current = current->next);

		current->next = tr;
	}
}

int
Text::execute(GraphicDevice *gd, Matrix *matrix, Cxform *cxform)
{
	TextRecord	*tr;
	long		 x,y;		// Current position
	SwfFont		*font = 0;	// Current font
	long		 fontHeight;
	Matrix		 tmat,fmat;

	x = y = 0;
	fontHeight = 0;

	// Compute final text matrix
	tmat = (*matrix) * textMatrix;

	for(tr = textRecords; tr; tr = tr ->next)
	{
		if (tr->flags & isTextControl) {
			if (tr->flags & textHasFont) {
				font = tr->font;
				fontHeight = tr->fontHeight;
				// Update font matrix
				fmat.a = fontHeight/1000.0;
				fmat.d = fontHeight/1000.0;
			}
			if (tr->flags & textHasXOffset) {
				x = tr->xOffset;
			}
			if (tr->flags & textHasYOffset) {
				y = tr->yOffset;
			}
			if (tr->flags & textHasColor) {
				if (cxform) {
					gd->setForegroundColor(cxform->getColor(tr->color));
				} else {
					gd->setForegroundColor(tr->color);
				}
			}
		} else {
			long g;

			assert(font != 0);
			for (g = 0; g < tr->nbGlyphs; g++)
			{
				Shape *shape;
				Matrix cmat;

				shape = font->getGlyph( tr->glyphs[g].index );

#ifdef PRINT
				printf("%c", font->getGlyphCode(tr->glyphs[g].index));
#endif

				// Update font matrix
				fmat.tx = x;
				fmat.ty = y;

				// Compute Character matrix
				cmat = tmat * fmat;

				shape->execute(gd, &cmat, cxform);

				// Advance
				x += tr->glyphs[g].xAdvance;
			}
#ifdef PRINT
			printf("\n");
#endif
		}
	}
	return 0;
}

void
Text::getRegion(GraphicDevice *gd, Matrix *m, unsigned char id)
{
}
